/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.intercept;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Iterator;

class InterceptorBuilder
{
    /**
     * interfaze Class -> MethodInterceptMatrix
     */
    private final HashMap matrices;

    /**
     * Object[] { StrictSignature, interceptor}
     */
    private final List intercepts;

    public InterceptorBuilder()
    {
        this.matrices = new HashMap();
        this.intercepts = new LinkedList();
    }

    /**
     * TODO Remove instanceof
     */
    public void register(Class interfaze)
    {
        MethodInterceptMatrix matrix = new MethodInterceptMatrix(interfaze);
        this.matrices.put(interfaze, matrix);
        for (Iterator i = this.intercepts.iterator(); i.hasNext();)
        {
            Object[] tuple = (Object[]) i.next();
            if (tuple[1] instanceof Class)
            {
                matrix.consider((Signature)tuple[0], (Class)tuple[1]);
            }
            else
            {
                matrix.consider((Signature)tuple[0], (Interceptor)tuple[1]);
            }
        }

    }

    public synchronized void addInterceptor(Signature signature, Class interceptor)
    {
        this.intercepts.add(new Object[] {signature, interceptor});
        for (Iterator i = this.matrices.keySet().iterator(); i.hasNext();)
        {
            MethodInterceptMatrix matrix = (MethodInterceptMatrix) this.matrices.get(i.next());
            matrix.consider(signature, interceptor);
        }
    }

    public synchronized void addInterceptor(Signature sig, Interceptor intercept)
    {
        this.intercepts.add(new Object[] {sig, intercept});
        for (Iterator i = this.matrices.keySet().iterator(); i.hasNext();)
        {
            MethodInterceptMatrix matrix = (MethodInterceptMatrix) this.matrices.get(i.next());
            matrix.consider(sig, intercept);
        }
    }

    public InterceptionHandler build(Class interfaze, Object base)
    {
        MethodInterceptMatrix matrix = (MethodInterceptMatrix) this.matrices.get(interfaze);
        return new InterceptionHandler(base, matrix);
    }
}
