package org.skife.intercept.interceptors;

import org.skife.intercept.Interceptor;
import org.skife.intercept.InterceptorStack;

import java.lang.reflect.Method;

/**
 * Throws an <code>UnsupportedOperationException</code>. Typically used
 * with an interceptor to prevent specific calls, for instance:
 * <pre><code>
 * InterceptionBroker broker = new InterceptionBroker();
 * broker.addInterceptor(new BlockingInterceptor("Cannot modify locked JavaBean"), new Signature()
 * {
 *     public boolean matches(Method m)
 *     {
 *         return (m.getName().startsWith("set") ? true : false);
 *     }
 * });
 *
 * // Create a wrapped, javabean-setter blocked version of foo. Note that foo can
 * // still be modified, but unmodifiableFoo cannot have setXxx(...) called
 * Foo unmodifiableFoo = (Foo) broker.wrap(foo);
 * </code></pre>
 */
public class BlockingInterceptor implements Interceptor
{
    /**
     * exception message
     */
    private String msg;

    /**
     * Create a new BlockingInterceptor which will throw an exception with the
     * message set to the argument
     *
     * @param msg Message for the <code>UnsupportedOperationException</code>
     */
    public BlockingInterceptor(String msg)
    {
        this.msg = msg;
    }

    /**
     * Throws an <code>UnsupportedOperationException</code>
     */
    public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack)
            throws Throwable
    {
        throw new UnsupportedOperationException(msg);
    }
}
