/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.intercept.interceptors;

import org.skife.intercept.Interceptor;
import org.skife.intercept.InterceptorStack;

import java.lang.reflect.Method;
import java.io.PrintStream;

/**
 * Logging interceptor that simply prints messages to a PrintStream such as
 * <code>System.out</code>
 */
public class InefficientLoggingInterceptor implements Interceptor
{
    private PrintStream out;

    /**
     * Send logging output to <code>System.out</code>
     */
    public InefficientLoggingInterceptor()
    {
        this(System.out);
    }

    /**
     * @param out PrintStream to log to
     */
    public InefficientLoggingInterceptor(PrintStream out)
    {
        this.out = out;
    }

    /**
     * Prints to <code>out</code> before and after invoking the stack
     */
    public Object intercept(Object base, Method method, Object[] args, InterceptorStack stack) throws Throwable
    {
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < args.length; ++i)
        {
            buffer.append(args[i].getClass().toString() + ": " + args[i]);
            if (!(i == args.length -1)) buffer.append(", ");
        }
        out.println(method.getDeclaringClass().getName() +
                    "#" +
                    method.getName() + " invoked with args: {" + buffer.toString() + "}");
        Object value = stack.yield();
        out.println(method.getDeclaringClass().getName() +
                    "#" +
                    method.getName() + " returned: " + value);
        return value;
    }
}
