/**
 * Copyright (c) 2003, Brian McCallister
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.skife.intercept;

import junit.framework.Assert;
import junit.framework.TestCase;
import org.skife.intercept.interceptors.InefficientLoggingInterceptor;
import org.skife.intercept.signatures.StrictSignature;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

public class TestInterceptionBroker extends TestCase
{
    public void testWrapBeforeDeclaringInterceptors() throws Exception
    {
        InterceptionBroker broker = new InterceptionBroker();
        MockInterceptor mi = new MockInterceptor();
        HashMap map = new HashMap();
        Map same = (Map) broker.wrap(map);
        broker.addInterceptor(mi, new StrictSignature(map.getClass(),
                                                "put",
                                                new Class[]{Object.class, Object.class},
                                                Object.class));

        same.put("thing", "value");
        Assert.assertTrue(mi.value);
        Assert.assertNotNull(same.get("thing"));
    }

    public void testWrapAfterDeclaringIntercepts() throws Exception
    {
        InterceptionBroker broker = new InterceptionBroker();
        MockInterceptor mi = new MockInterceptor();
        broker.addInterceptor(mi, new StrictSignature(HashMap.class,
                                                "put",
                                                new Class[]{Object.class, Object.class},
                                                Object.class));
        HashMap map = new HashMap();
        Map same = (Map) broker.wrap(map);
        same.put("thing", "value");
        Assert.assertTrue(mi.value);
        Assert.assertNotNull(same.get("thing"));
    }

    /**
     * Disabled as it requires human validation of stdout
     */
    public void _testLogging() throws Exception
    {
        InterceptionBroker broker = new InterceptionBroker();
        Interceptor mi = new InefficientLoggingInterceptor();
        broker.addInterceptor(mi, new StrictSignature(HashMap.class,
                                                "put",
                                                new Class[]{Object.class, Object.class},
                                                Object.class));
        HashMap map = new HashMap();
        Map same = (Map) broker.wrap(map);
        same.put("thing", "value");
        Assert.assertNotNull(same.get("thing"));
    }

    public void testWrapInterceptedObject() throws Exception
    {
        InterceptionBroker broker = new InterceptionBroker();
        HashMap one = new HashMap();
        HashMap wrapped = (HashMap) broker.wrap(one);
        MockInterceptor mock = new MockInterceptor();
        broker.addInterceptor(mock, new Signature()
        {
            public boolean matches(Method m)
            {
                return true;
            }
        });
        HashMap again = (HashMap) broker.wrap(wrapped);
        again.put("a", "b");
        assertTrue(mock.value);
    }
}
